[llvm-commits] [llvm] r77397 - in /llvm/trunk: include/llvm/Support/FormattedStream.h lib/Support/FormattedStream.cpp

David Greene greened at obbligato.org
Tue Jul 28 16:26:34 PDT 2009


Author: greened
Date: Tue Jul 28 18:26:34 2009
New Revision: 77397

URL: http://llvm.org/viewvc/llvm-project?rev=77397&view=rev
Log:

Improve performance of PadToColumn by eliminating flushes.

Modified:
    llvm/trunk/include/llvm/Support/FormattedStream.h
    llvm/trunk/lib/Support/FormattedStream.cpp

Modified: llvm/trunk/include/llvm/Support/FormattedStream.h
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/include/llvm/Support/FormattedStream.h?rev=77397&r1=77396&r2=77397&view=diff

==============================================================================
--- llvm/trunk/lib/Support/FormattedStream.cpp (original)
+++ llvm/trunk/lib/Support/FormattedStream.cpp Tue Jul 28 18:26:34 2009
@@ -19,11 +19,11 @@
 /// ComputeColumn - Examine the current output and figure out which
 /// column we end up in after output.
 ///
-void formatted_raw_ostream::ComputeColumn(const char *Ptr, size_t Size) {
+void formatted_raw_ostream::ComputeColumn(unsigned &Column) {
   // Keep track of the current column by scanning the string for
   // special characters
 
-  for (const char *epos = Ptr + Size; Ptr != epos; ++Ptr) {
+  for (const char *Ptr = begin(); Ptr != end(); ++Ptr) {
     ++Column;
     if (*Ptr == '\n' || *Ptr == '\r')
       Column = 0;
@@ -38,8 +38,13 @@
 /// \param MinPad - The minimum space to give after the most recent
 /// I/O, even if the current column + minpad > newcol.
 ///
-void formatted_raw_ostream::PadToColumn(unsigned NewCol, unsigned MinPad) {
-  flush();
+void formatted_raw_ostream::PadToColumn(unsigned NewCol, unsigned MinPad) { 
+  // Start out from the last flush position.
+  unsigned Column = ColumnFlushed;
+
+  // Now figure out what's in the buffer and add it to the column
+  // count.
+  ComputeColumn(Column);
 
   // Output spaces until we reach the desired column.
   unsigned num = NewCol - Column;





More information about the llvm-commits mailing list
--- llvm/trunk/include/llvm/Support/FormattedStream.h (original)
+++ llvm/trunk/include/llvm/Support/FormattedStream.h Tue Jul 28 18:26:34 2009
@@ -49,13 +49,13 @@
     ///
     bool DeleteStream;
 
     /// Column - The current output column of the stream.  The column
     /// scheme is zero-based.
     ///
-    unsigned Column;
+    unsigned ColumnFlushed;
 
     virtual void write_impl(const char *Ptr, size_t Size) {
-      ComputeColumn(Ptr, Size);
+      ComputeColumn(ColumnFlushed);
       TheStream->write(Ptr, Size);
     }
 
@@ -67,10 +67,10 @@
       return TheStream->tell() - TheStream->GetNumBytesInBuffer();
     }
 
-    /// ComputeColumn - Examine the current output and figure out
-    /// which column we end up in after output.
+    /// ComputeColumn - Examine the current buffer and figure out
+    /// which column we're in.
     ///
-    void ComputeColumn(const char *Ptr, size_t Size);
+    void ComputeColumn(unsigned &Column);
 
   public:
     /// formatted_raw_ostream - Open the specified file for
@@ -84,11 +84,11 @@
     /// underneath it.
     ///
     formatted_raw_ostream(raw_ostream &Stream, bool Delete = false) 
-      : raw_ostream(), TheStream(0), DeleteStream(false), Column(0) {
+      : raw_ostream(), TheStream(0), DeleteStream(false), ColumnFlushed(0) {
       setStream(Stream, Delete);
     }
     explicit formatted_raw_ostream()
-      : raw_ostream(), TheStream(0), DeleteStream(false), Column(0) {}
+      : raw_ostream(), TheStream(0), DeleteStream(false), ColumnFlushed(0) {}
 
     ~formatted_raw_ostream() {
       if (DeleteStream)
@@ -49,13 +49,13 @@
     bool DeleteStream;
 
-    /// Column - The current output column of the stream.  The column
-    /// scheme is zero-based.
+    /// ColumnFlushed - The current output column of the data that's
+    /// been flushed.  The column scheme is zero-based.
     ///

Modified: llvm/trunk/lib/Support/FormattedStream.cpp
URL: http://llvm.org/viewvc/llvm-project/llvm/trunk/lib/Support/FormattedStream.cpp?rev=77397&r1=77396&r2=77397&view=diff

==============================================================================